function [out]=SeqSearchRandLleading(ID_RTvector)
% Searches for 1 to 5 trial button-press sequences. Copyright 2022 by Jay Holden
% Assumes a 1600X2 column vector, with column one containing R-presses coded as 1 and L-presses coded as zeros. Column 2 contains Response times. IF RT is not analyzed, it can be fixed to a constant millisecond value, such as 500.
% e.g.: [1 500
%        1 350
%        0 450
%        1 550
%        0 324];
% It outputs a matrix in which the permutations run down the rows, and all sequences are listed across the columns. That means each col. length has to be divided by the sequence length to get the number of sequences. So a col. length of 25 for a 5-trial sequence contains RTs for 5 sequences. A col. length of 20 for a 2-trial sequence contains 10 sequences.
% computes all 62 permutations of a Stern-Brocot tree, from 1/6 to 6.

%These are the sequence lengths, starting with RRRRR and ending with LLLLL.
slen=[5	4	5	3	5	4	5	2	5	4	5	3	5	4	5	1	5	4	5	3	5	4	5	2	5	4	5	3	5	4	5];
rlslen=repmat(slen,1,2)';

%preallocate output matrix
out=NaN(length(rlslen),round(length(ID_RTvector)*.75));
%Preallocating the output matrix, which places the RTs for identified sequence
    %in succession down the col.s in M, in the row corresponding to the searched
    %token. It is a 62 Rows X 3/4 the length of the RT series. If biased
    %sequences are searched, it may append extra zeros if it is lengthened
    %automatically by Matlab. Appropriate checks should be built into analysis code

%insert the Rs and Ls
justR=ID_RTvector(ID_RTvector(:,1)==1,2)'; out(16,1:length(justR))=justR;
justL=ID_RTvector(ID_RTvector(:,1)==0,2)'; out(47,1:length(justL))=justL;

%compute SB tree
totalDepth=5;
for L=2:totalDepth
    curDepth=L;
    lo=2;
    hi=3;

        for K=2:curDepth

            M=dec2bin(lo:hi);
            M=double(M)-48;
            M=flipud(M);

            lo=hi+1;
            hi=lo*2-1;
        end
    ML=flipud(-1*M+1);
    MR=M;
    M=[MR;ML];
    
    haystack=ID_RTvector(:,1);
    %haystack is the 1-0 col. vector of trial identities
    RT=ID_RTvector(:,2);
    %RT is the col. vector containing the +/- RT measuremets.
    %The M vector defines the 31 search codes. They are ordered from
    %the smallest mode-locking ratio, to the largest (1/6 to 5/6).
    %haystack =randi([0 1],1000,1); 
    %haystack=repmat([1 0 0 0 0]',100,1);
    %commented testing code, generates various 1-0 vectors checking performance.
    matIndex=find(rlslen==size(M,2));
    %Preallocating the output matrix, which places the RTs for identified sequence
    %in succession down the col.s in M, in the row corresponding to the searched
    %token. It is a 31 Rows X 3/4 the length of the RT series. If biased
    %sequences are searched, it may append extra zeros if it is lengthened
    %automatically by Matlab. Appropriate checks should be built into analysis code.

    for i=1:length(M)
        % R=1, L=0.
        %sequence is the "needle" in a "haystack" of ones and zeros.
            needle=M(i,:); %Get 0 1 sequence from M.
            if all(needle==0)
               needle=needle+1;
               haystack=-1.*haystack+1;
               %special case for all L's, matrix ops do not work on all
               %zeros. They are switched to all ones.
            else
            end

            A=zeros(length(haystack));
            %Preallocate an empty square matrix. 
            %Insert the haystack as the first full col.
            A(:,1)=haystack;


                for j=2:length(A); A(:,j)=circshift(haystack,j-1,1); end
                %lag embed the haystack, with delay = 1, down the cols of A.

            A=tril(A);
            %Pull the lower diag of A, which sets the upper diag to zeros.
            %Now every sequence is a row, with its begining as its row address.
            %3-trial sequences are 3 cols, 5 trials are 5 cols etc.
            seq=A(:,1:length(needle)); seq=fliplr(seq);
            %Flip so rows in cardinal increasing order.

            seq(1:(length(needle)-1),:)=[];
            % generate a copy of the pattern that is as long as the searched
            % sequence of data (The Haystack).

            repNeedle=repmat(needle,length(seq),1);
            %subtract the desired sequence from every row. Needs at least one 1.
            %can't search for all zeros, because subtraction will not change values

            result=repNeedle-seq; result=result*result';
            %Then multiply it by its own transpose, all-zero row/cols index start
            %of pattern match.


            [R,~]=find(sum(result,2)==0);
            %search for the begining address of all matches, gives a count
        %end
                if isempty(R)
                    continue %if nothing found, move to next pattern.
                else
                    ctr=1;
                    for k=1:length(R)
                    out(matIndex(i),ctr:ctr+length(needle)-1)=RT(R(k):(R(k)+length(needle)-1),1);
                    ctr=ctr+length(needle);
                   %pull the RTs corresponding to each sequence, place RTs of
                   %succesive finds in appropriate col. of out matrix. 
                    end 
                end


    end
    
end

